package org.jboss.cache.api.batch;

import org.jboss.cache.Cache;
import org.jboss.cache.CacheFactory;
import org.jboss.cache.UnitTestCacheFactory;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.config.Configuration.NodeLockingScheme;
import org.jboss.cache.config.ConfigurationException;
import org.jboss.cache.util.TestingUtil;
import org.testng.annotations.Test;

@Test(groups = "functional", testName = "api.batch.BatchWithoutTM")
public class BatchWithoutTM extends AbstractBatchTest
{
   public void testBatchWithoutCfg()
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache(false);
         try
         {
            cache.startBatch();
            assert false : "Should have failed";
         }
         catch (ConfigurationException good)
         {
            // do nothing
         }

         try
         {
            cache.endBatch(true);
            assert false : "Should have failed";
         }
         catch (ConfigurationException good)
         {
            // do nothing
         }

         try
         {
            cache.endBatch(false);
            assert false : "Should have failed";
         }
         catch (ConfigurationException good)
         {
            // do nothing
         }
      }
      finally
      {
         TestingUtil.killCaches(cache);
         cache = null;
      }
   }

   public void testEndBatchWithoutStartBatch()
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache(true);
         cache.endBatch(true);
         cache.endBatch(false);
         // should not fail.
      }
      finally
      {
         TestingUtil.killCaches(cache);
      }
   }

   public void testStartBatchIdempotency()
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache(true);
         cache.startBatch();
         cache.put("/a/b/c", "k", "v");
         cache.startBatch();     // again
         cache.put("/a/b/c", "k2", "v2");
         cache.endBatch(true);

         assert "v".equals(cache.get("/a/b/c", "k"));
         assert "v2".equals(cache.get("/a/b/c", "k2"));
      }
      finally
      {
         TestingUtil.killCaches(cache);
         cache = null;
      }
   }

   public void testBatchVisibility() throws InterruptedException
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache(true);
         cache.startBatch();
         cache.put("/a/b/c", "k", "v");
         assert getOnDifferentThread(cache, "/a/b/c", "k") == null : "Other thread should not see batch update till batch completes!";

         cache.endBatch(true);

         assert "v".equals(getOnDifferentThread(cache, "/a/b/c", "k"));
      }
      finally
      {
         TestingUtil.killCaches(cache);
         cache = null;
      }
   }

   public void testBatchRollback() throws Exception
   {
      Cache<String, String> cache = null;
      try
      {
         cache = createCache(true);
         cache.startBatch();
         cache.put("/a/b/c", "k", "v");
         cache.put("/a/b/c", "k2", "v2");

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;

         cache.endBatch(false);

         assert getOnDifferentThread(cache, "/a/b/c", "k") == null;
         assert getOnDifferentThread(cache, "/a/b/c", "k2") == null;
      }
      finally
      {
         TestingUtil.killCaches(cache);
         cache = null;
      }
   }

   private Cache<String, String> createCache(boolean enableBatch)
   {
      CacheFactory<String, String> cf = new UnitTestCacheFactory<String, String>();
      Configuration c = new Configuration();
      c.setNodeLockingScheme(NodeLockingScheme.MVCC);
      c.setInvocationBatchingEnabled(enableBatch);
      return cf.createCache(c);
   }
}
